@@ -2348,23 +2348,36 @@ static int demux_ts_get_optional_data(demux_plugin_t *this_gen,
     {
     case DEMUX_OPTIONAL_DATA_AUDIOLANG:
       if ((channel >= 0) && (channel < this->audio_tracks_count)) {
-        if(this->audio_tracks[channel].lang)
+        if (this->audio_tracks[channel].lang[0]) {
           strcpy(str, this->audio_tracks[channel].lang);
-        else
-          sprintf(str, "%3i", _x_get_audio_channel(this->stream));
+        } else {
+          /* input plugin may know the language */
+          if (this->input->get_capabilities(this->input) & INPUT_CAP_AUDIOLANG)
+            return DEMUX_OPTIONAL_UNSUPPORTED;
+          sprintf(str, "%3i", channel);
+        }
+        return DEMUX_OPTIONAL_SUCCESS;
       }
       else {
-        snprintf(str, XINE_LANG_MAX, "%3i", _x_get_audio_channel(this->stream));
+        strcpy(str, "none");
       }
-      return DEMUX_OPTIONAL_SUCCESS;
+      return DEMUX_OPTIONAL_UNSUPPORTED;
 
     case DEMUX_OPTIONAL_DATA_SPULANG:
       if (channel>=0 && channel<this->spu_langs_count) {
-        memcpy(str, this->spu_langs[channel].desc.lang, 3);
-	str[3] = 0;}
-      else
+        if (this->spu_langs[channel].desc.lang[0]) {
+          strcpy(str, this->spu_langs[channel].desc.lang);
+        } else {
+          /* input plugin may know the language */
+          if (this->input->get_capabilities(this->input) & INPUT_CAP_SPULANG)
+            return DEMUX_OPTIONAL_UNSUPPORTED;
+          sprintf(str, "%3i", channel);
+        }
+        return DEMUX_OPTIONAL_SUCCESS;
+      } else {
         strcpy(str, "none");
-      return DEMUX_OPTIONAL_SUCCESS;
+      }
+      return DEMUX_OPTIONAL_UNSUPPORTED;
 
     default:
       return DEMUX_OPTIONAL_UNSUPPORTED;
