@@ -26,6 +26,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <errno.h>
 
 #define XINE_ENGINE_INTERNAL
 
@@ -102,6 +103,11 @@ int _x_spu_decoder_sleep(xine_stream_t *stream, int64_t next_spu_vpts)
   return thread_vacant;
 }
 
+static void video_decoder_update_disable_flush_at_discontinuity(void *disable_decoder_flush_at_discontinuity, xine_cfg_entry_t *entry)
+{
+  *(int *)disable_decoder_flush_at_discontinuity = entry->num_value;
+}
+
 static void *video_decoder_loop (void *stream_gen) {
 
   buf_element_t   *buf;
@@ -112,14 +118,12 @@ static void *video_decoder_loop (void *stream_gen) {
   int              prof_video_decode = -1;
   int              prof_spu_decode = -1;
   uint32_t         buftype_unknown = 0;
+  int              disable_decoder_flush_at_discontinuity;
 
 #ifndef WIN32
-  /* nice(-value) will fail silently for normal users.
-   * however when running as root this may provide smoother
-   * playback. follow the link for more information:
-   * http://cambuca.ldhs.cetuc.puc-rio.br/~miguel/multimedia_sim/
-   */
-  nice(-1);
+  errno = 0;
+  if (nice(-1) == -1 && errno)
+    xine_log(stream->xine, XINE_LOG_MSG, "video_decoder: can't raise nice priority by 1: %s\n", strerror(errno));
 #endif /* WIN32 */
 
   if (prof_video_decode == -1)
@@ -127,6 +131,15 @@ static void *video_decoder_loop (void *stream_gen) {
   if (prof_spu_decode == -1)
     prof_spu_decode = xine_profiler_allocate_slot ("spu decoder");
 
+  disable_decoder_flush_at_discontinuity = stream->xine->config->register_bool(stream->xine->config, "engine.decoder.disable_flush_at_discontinuity", 0,
+      _("disable decoder flush at discontinuity"),
+      _("when watching live tv a discontinuity happens for example about every 26.5 hours due to a pts wrap.\n"
+        "flushing the decoder at that time causes decoding errors for images after the pts wrap.\n"
+        "to avoid the decoding errors, decoder flush at discontinuity should be disabled.\n\n"
+        "WARNING: as the flush was introduced to fix some issues when playing DVD still images, it is\n"
+        "likely that these issues may reappear in case they haven't been fixed differently meanwhile.\n"),
+        20, video_decoder_update_disable_flush_at_discontinuity, &disable_decoder_flush_at_discontinuity);
+
   while (running) {
 
     lprintf ("getting buffer...\n");
@@ -307,10 +320,11 @@ static void *video_decoder_loop (void *stream_gen) {
 
       if (stream->video_decoder_plugin) {
         running_ticket->acquire(running_ticket, 0);
-        /* it might be a long time before we get back from a discontinuity, so we better flush
-	 * the decoder before */
-        stream->video_decoder_plugin->flush (stream->video_decoder_plugin);
         stream->video_decoder_plugin->discontinuity (stream->video_decoder_plugin);
+        /* it might be a long time before we get back from a handle_video_discontinuity,
+	 * so we better flush the decoder before */
+        if (!disable_decoder_flush_at_discontinuity)
+          stream->video_decoder_plugin->flush (stream->video_decoder_plugin);
         running_ticket->release(running_ticket, 0);
       }
 
@@ -323,10 +337,11 @@ static void *video_decoder_loop (void *stream_gen) {
 
       if (stream->video_decoder_plugin) {
         running_ticket->acquire(running_ticket, 0);
-        /* it might be a long time before we get back from a discontinuity, so we better flush
-	 * the decoder before */
-        stream->video_decoder_plugin->flush (stream->video_decoder_plugin);
         stream->video_decoder_plugin->discontinuity (stream->video_decoder_plugin);
+        /* it might be a long time before we get back from a handle_video_discontinuity,
+	 * so we better flush the decoder before */
+        if (!disable_decoder_flush_at_discontinuity)
+          stream->video_decoder_plugin->flush (stream->video_decoder_plugin);
         running_ticket->release(running_ticket, 0);
       }
 
@@ -350,6 +365,19 @@ static void *video_decoder_loop (void *stream_gen) {
     case BUF_CONTROL_NOP:
       break;
 
+    case BUF_CONTROL_RESET_TRACK_MAP:
+      if (stream->spu_track_map_entries)
+      {
+        xine_event_t ui_event;
+
+        stream->spu_track_map_entries = 0;
+
+        ui_event.type        = XINE_EVENT_UI_CHANNELS_CHANGED;
+        ui_event.data_length = 0;
+        xine_event_send(stream, &ui_event);
+      }
+      break;
+
     default:
 
       if ( (buf->type & 0xFF000000) == BUF_VIDEO_BASE ) {
